div#portal-column-one div.portletWrapper dl.portletGoToParent {
    margin-bottom:0em;
}

/* Disable simplelayout design menu on chapter, since we cannot
   create a PDF with multiple columns. */
.portaltype-chapter #simplelayout-contentmenu-design {
  display: none;
}

.BlockOverallWrapper.chapter > a {
  display: block;
  margin-bottom: 2em;
}

.BlockOverallWrapper.remark {
  margin: 10px 0;
  padding: 5px;
  padding-bottom: 6px;

  -webkit-border-radius: 10px;
  -moz-border-radius: 10px;
  -o-border-radius: 10px;
  -ms-border-radius: 10px;
  -khtml-border-radius: 10px;
  border-radius: 10px;

  background: #eeeeee; /* Old browsers */
  /* IE9 SVG, needs conditional override of 'filter' to 'none' */
  background: url(data:image/svg+xml;base64,PD94bWwgdmVyc2lvbj0iMS4wIiA/Pgo8c3ZnIHhtbG5zPSJodHRwOi8vd3d3LnczLm9yZy8yMDAwL3N2ZyIgd2lkdGg9IjEwMCUiIGhlaWdodD0iMTAwJSIgdmlld0JveD0iMCAwIDEgMSIgcHJlc2VydmVBc3BlY3RSYXRpbz0ibm9uZSI+CiAgPGxpbmVhckdyYWRpZW50IGlkPSJncmFkLXVjZ2ctZ2VuZXJhdGVkIiBncmFkaWVudFVuaXRzPSJ1c2VyU3BhY2VPblVzZSIgeDE9IjAlIiB5MT0iMCUiIHgyPSIwJSIgeTI9IjEwMCUiPgogICAgPHN0b3Agb2Zmc2V0PSIwJSIgc3RvcC1jb2xvcj0iI2VlZWVlZSIgc3RvcC1vcGFjaXR5PSIxIi8+CiAgICA8c3RvcCBvZmZzZXQ9IjEwMCUiIHN0b3AtY29sb3I9IiNjY2NjY2MiIHN0b3Atb3BhY2l0eT0iMSIvPgogIDwvbGluZWFyR3JhZGllbnQ+CiAgPHJlY3QgeD0iMCIgeT0iMCIgd2lkdGg9IjEiIGhlaWdodD0iMSIgZmlsbD0idXJsKCNncmFkLXVjZ2ctZ2VuZXJhdGVkKSIgLz4KPC9zdmc+);
  background: -moz-linear-gradient(top,  #eeeeee 0%, #cccccc 100%); /* FF3.6+ */
  background: -webkit-gradient(linear, left top, left bottom, color-stop(0%,#eeeeee), color-stop(100%,#cccccc)); /* Chrome,Safari4+ */
  background: -webkit-linear-gradient(top,  #eeeeee 0%,#cccccc 100%); /* Chrome10+,Safari5.1+ */
  background: -o-linear-gradient(top,  #eeeeee 0%,#cccccc 100%); /* Opera 11.10+ */
  background: -ms-linear-gradient(top,  #eeeeee 0%,#cccccc 100%); /* IE10+ */
  background: linear-gradient(top,  #eeeeee 0%,#cccccc 100%); /* W3C */
  filter: progid:DXImageTransform.Microsoft.gradient( startColorstr='#eeeeee', endColorstr='#cccccc',GradientType=0 ); /* IE6-8 */

}

.BlockOverallWrapper.remark.blockHighlight {
    padding-bottom: 5px;
}

/* Table content-type */

.simplelayout-content table {
    width: 100%;
}
/*table.bibliothekTable th {
    font-weight: normal;
}
*/
.border-grid {
    border: 1px solid #DEDEDE;
}
.border-right {
    border-right:1px solid #DEDEDE;
}
.border-left {
    border-left:1px solid #DEDEDE;
}
.border-bottom {
    border-bottom-width:1px;
    border-bottom-color: #DEDEDE;
    border-bottom-style:solid;
}
.border-top {
    border-top-width:1px;
    border-top-color:#DEDEDE;
    border-top-style:solid;
}
.right {
    text-align: right;
}
.left {
    text-align: left;
}
.center {
    text-align: center;
}
.indent2{
    padding-left: 2mm;
}
.indent10{
    padding-left: 10mm;
}
.bold {
    font-weight: bold !important;
}
.grey {
    color: #808080;
}
.scriptsize {
    font-size: 7.5pt;
    font-weight: normal;
}
.footnotesize {
    font-size: 7pt;
}

.sl-export-import-action {
  background: url(++resource++ftw.book-resources/icon_export_import.png) no-repeat !important;
}


/** TINYMCE keyword plugin **/
.ploneSkin span.mce_tinymce_keyword {
	background: url("++resource++ftw.book.tinymce.keyword/img/keyword.png") no-repeat center center;
}

span.keyword, body.mceContentBody span.keyword {
  border: 1px dashed green;
  padding-left: 1px;
  padding-right: 1px;
}

/** TINYMCE footnote plugin **/
.ploneSkin span.mce_tinymce_footnote {
  background: url("++resource++ftw.book.tinymce.footnote/img/footnote.png") no-repeat center center;
}

span.footnote:after {
  display: inline-block;
  content: ' ';
  background-image: url(++resource++ftw.book.tinymce.footnote/img/info.png);
  background-size: contain;
  background-repeat: no-repeat;
  margin: 0 2px;
  width: 1em;
  height: 1em;
}
span.footnote {
  position: relative;
}
span.footnote:hover:before {
  border: 1px solid #205c90;
  padding: 3px 6px;
  background: #ddd;
  content: attr(data-footnote);
  position: absolute;
  left: 0;
  top: 1.3em;
  width: 200px;
  text-align: center;
  z-index: 2;
}

/* Keywords tab */
#keywords_overview {
  padding-top: 1em;
}

#keywords_overview form .select2-container {
  float: left;
  margin-right: 0.5em;
}

#keywords_overview .result,
#keywords_overview .no-results {
  clear: both;
  padding-top: 1em;
  margin-bottom: 1em;
}
